@@ -11,23 +11,24 @@
 __all__ = [ 'Port' ]
 
 import logging, sys
-from six import iteritems
 from weakref import ref as weakref_ref
 
 from pyomo.common.collections import ComponentMap
-from pyomo.common.timing import ConstructionTimer
+from pyomo.common.deprecation import RenamedClass
+from pyomo.common.formatting import tabular_writer
+from pyomo.common.log import is_debug_set
 from pyomo.common.modeling import unique_component_name
+from pyomo.common.timing import ConstructionTimer
 
 from pyomo.core.base.var import Var
 from pyomo.core.base.constraint import Constraint
-from pyomo.core.base.component import ComponentData
+from pyomo.core.base.component import ComponentData, ModelComponentFactory
 from pyomo.core.base.indexed_component import \
     IndexedComponent, UnindexedComponent_set
-from pyomo.core.base.misc import apply_indexed_rule, tabular_writer
+from pyomo.core.base.misc import apply_indexed_rule
 from pyomo.core.base.numvalue import as_numeric, value
 from pyomo.core.expr.current import identify_variables
 from pyomo.core.base.label import alphanum_label_from_name
-from pyomo.core.base.plugin import ModelComponentFactory
 
 from pyomo.network.util import create_var, tighten_var_domain
 
@@ -66,8 +67,18 @@ class _PortData(ComponentData):
         state = super(_PortData, self).__getstate__()
         for i in _PortData.__slots__:
             state[i] = getattr(self, i)
+
+        # Remove/resolve weak references
+        for i in ('_arcs', '_sources', '_dests'):
+            state[i] = [ref() for ref in state[i]]
         return state
 
+    def __setstate__(self, state):
+        state['_arcs'] = [weakref_ref(i) for i in state['_arcs']]
+        state['_sources'] = [weakref_ref(i) for i in state['_sources']]
+        state['_dests'] = [weakref_ref(i) for i in state['_dests']]
+        super(_PortData, self).__setstate__(state)
+
     # Note: None of the slots on this class need to be edited, so we
     # don't need to implement a specialized __setstate__ method, and
     # can quietly rely on the super() class's implementation.
@@ -236,12 +247,12 @@ class _PortData(ComponentData):
             names: `bool`
                 If True, yield (name, index, var/expr) tuples
         """
-        for name, mem in iteritems(self.vars):
+        for name, mem in self.vars.items():
             if not mem.is_indexed():
                 itr = {None: mem}
             else:
                 itr = mem
-            for idx, v in iteritems(itr):
+            for idx, v in itr.items():
                 if fixed is not None and v.is_fixed() != fixed:
                     continue
                 if expr_vars and v.is_expression_type():
@@ -316,7 +327,7 @@ class Port(IndexedComponent):
         if cls != Port:
             return super(Port, cls).__new__(cls)
         if not args or (args[0] is UnindexedComponent_set and len(args) == 1):
-            return SimplePort.__new__(SimplePort)
+            return ScalarPort.__new__(ScalarPort)
         else:
             return IndexedPort.__new__(IndexedPort)
 
@@ -336,7 +347,7 @@ class Port(IndexedComponent):
         return tmp
 
     def construct(self, data=None):
-        if __debug__ and logger.isEnabledFor(logging.DEBUG):  #pragma:nocover
+        if is_debug_set(logger):  #pragma:nocover
             logger.debug( "Constructing Port, name=%s, from data=%s"
                           % (self.name, data) )
 
@@ -367,7 +378,7 @@ class Port(IndexedComponent):
             for key in self._implicit:
                 tmp.add(None, key)
             if self._extends:
-                for key, val in iteritems(self._extends.vars):
+                for key, val in self._extends.vars.items():
                     tmp.add(val, key, self._extends.rule_for(key))
             if self._initialize:
                 self._add_from_container(tmp, self._initialize)
@@ -378,7 +389,7 @@ class Port(IndexedComponent):
 
     def _add_from_container(self, port, items):
         if type(items) is dict:
-            for key, val in iteritems(items):
+            for key, val in items.items():
                 if type(val) is tuple:
                     if len(val) == 2:
                         obj, rule = val
@@ -403,7 +414,7 @@ class Port(IndexedComponent):
     def _pprint(self, ostream=None, verbose=False):
         """Print component information."""
         def _line_generator(k, v):
-            for _k, _v in sorted(iteritems(v.vars)):
+            for _k, _v in sorted(v.vars.items()):
                 if _v is None:
                     _len = '-'
                 elif _v.is_indexed():
@@ -414,7 +425,7 @@ class Port(IndexedComponent):
         return (
             [("Size", len(self)),
              ("Index", self._index if self.is_indexed() else None)],
-             iteritems(self._data),
+             self._data.items(),
              ( "Name", "Size", "Variable"),
              _line_generator)
 
@@ -434,7 +445,7 @@ class Port(IndexedComponent):
 
         ostream.write("\n")
         def _line_generator(k,v):
-            for _k, _v in sorted(iteritems(v.vars)):
+            for _k, _v in sorted(v.vars.items()):
                 if _v is None:
                     _val = '-'
                 elif not _v.is_indexed():
@@ -445,7 +456,7 @@ class Port(IndexedComponent):
                             x, value(_v[x])) for x in sorted(_v._data)))
                 yield _k, _val
         tabular_writer(ostream, prefix+tab,
-                       ((k, v) for k, v in iteritems(self._data)),
+                       ((k, v) for k, v in self._data.items()),
                        ("Name", "Value"), _line_generator)
 
     @staticmethod
@@ -608,7 +619,7 @@ class Port(IndexedComponent):
             if eblock.component("splitfrac") is None:
                 if not include_splitfrac:
                     num_data_objs = 0
-                    for k, v in iteritems(port.vars):
+                    for k, v in port.vars.items():
                         if port.is_extensive(k):
                             if v.is_indexed():
                                 num_data_objs += len(v)
@@ -713,13 +724,18 @@ class Port(IndexedComponent):
             evar = create_var(member, name, eblock, index_set)
         return evar
 
-class SimplePort(Port, _PortData):
+class ScalarPort(Port, _PortData):
 
     def __init__(self, *args, **kwd):
         _PortData.__init__(self, component=self)
         Port.__init__(self, *args, **kwd)
 
 
+class SimplePort(metaclass=RenamedClass):
+    __renamed__new_class__ = ScalarPort
+    __renamed__version__ = '6.0'
+
+
 class IndexedPort(Port):
     pass
 
